@@ -35,8 +35,6 @@
 
 static const char *empty_error = "";
 
-zckCtx zck_none = {0};
-
 void set_error_wf(zckCtx *zck, int fatal, const char *function,
                   const char *format, ...) {
     va_list args;
@@ -45,16 +43,17 @@ void set_error_wf(zckCtx *zck, int fatal, const char *function,
     int old_size = 0;
     assert(format != NULL);
 
-    if(zck == NULL)
-        zck = &zck_none;
-
-    zck->error_state = 1 + (fatal > 0 ? 1 : 0);
     va_start(args, format);
     size = vsnprintf(NULL, 0, format, args);
     va_end(args);
     va_start(args, format);
     zck_log_v(function, ZCK_LOG_ERROR, format, args);
     va_end(args);
+
+    if(zck == NULL)
+        return;
+
+    zck->error_state = 1 + (fatal > 0 ? 1 : 0);
     if(size < 0)
         return;
     if(zck->msg != NULL) {
@@ -83,17 +82,15 @@ void set_error_wf(zckCtx *zck, int fatal, const char *function,
 
 int ZCK_PUBLIC_API zck_is_error(zckCtx *zck) {
     if(zck == NULL)
-        zck = &zck_none;
+        return 0;
 
     return zck->error_state;
 }
 
 const char ZCK_PUBLIC_API *zck_get_error(zckCtx *zck) {
-    if(zck == NULL)
-        zck = &zck_none;
-
-    if(zck->msg == NULL)
+    if(zck == NULL || zck->msg == NULL)
         return empty_error;
+
     return zck->msg;
 }
 
@@ -102,7 +99,7 @@ bool ZCK_PUBLIC_API zck_clear_error(zckCtx *zck) {
         return false;
 
     if(zck == NULL)
-        zck = &zck_none;
+        return true;
 
     free(zck->msg);
     zck->msg = NULL;
