@@ -32,6 +32,8 @@
 #include <sys/endian.h>
 #elif __APPLE__
 #include <machine/endian.h>
+#elif defined(_WIN32)
+// no endian.h available
 #else
 #include <endian.h>
 #endif
@@ -74,9 +76,18 @@ bool index_read(zckCtx *zck, char *data, size_t size, size_t max_length) {
 
         zckChunk *tmp = NULL;
         zckChunk *new = zmalloc(sizeof(zckChunk));
+        if (!new) {
+           zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+           return false;
+        }
 
         /* Read index entry digest */
         new->digest = zmalloc(zck->index.digest_size);
+        if (!new->digest) {
+           zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+           free(new);
+           return false;
+        }
         memcpy(new->digest, data+length, zck->index.digest_size);
         new->digest_size = zck->index.digest_size;
         HASH_FIND(hh, zck->index.ht, new->digest, new->digest_size, tmp);
@@ -85,6 +96,23 @@ bool index_read(zckCtx *zck, char *data, size_t size, size_t max_length) {
                             new);
         length += zck->index.digest_size;
 
+        /* Read uncompressed entry digest, if any */
+        if (zck->has_uncompressed_source) {
+            /* same size for digest as compressed */
+            new->digest_uncompressed = zmalloc(zck->index.digest_size);
+            if (!new->digest_uncompressed) {
+                free(new->digest);
+                free(new);
+                zck_log(ZCK_LOG_ERROR, "OOM in %s", __func__);
+                return false;
+            }
+            memcpy(new->digest_uncompressed, data+length, zck->index.digest_size);
+            HASH_FIND(hhuncomp, zck->index.htuncomp, new->digest_uncompressed, new->digest_size, tmp);
+            if(!tmp)
+               HASH_ADD_KEYPTR(hhuncomp, zck->index.htuncomp, new->digest_uncompressed, new->digest_size,
+                               new);
+            length += zck->index.digest_size;
+	}
         /* Read and store entry length */
         size_t chunk_length = 0;
         if(!compint_to_size(zck, &chunk_length, data+length, &length,
@@ -123,30 +151,34 @@ bool index_read(zckCtx *zck, char *data, size_t size, size_t max_length) {
     return true;
 }
 
-ssize_t PUBLIC zck_get_chunk_count(zckCtx *zck) {
+ssize_t ZCK_PUBLIC_API zck_get_chunk_count(zckCtx *zck) {
     VALIDATE_INT(zck);
 
     return zck->index.count;
 }
 
-zckChunk PUBLIC *zck_get_chunk(zckCtx *zck, size_t number) {
+zckChunk ZCK_PUBLIC_API *zck_get_chunk(zckCtx *zck, size_t number) {
     VALIDATE_PTR(zck);
 
     for(zckChunk *idx=zck->index.first; idx!=NULL; idx=idx->next) {
         if(idx->number == number)
             return idx;
     }
-    zck_log(ZCK_LOG_WARNING, "Chunk %lu not found", number);
+    zck_log(
+        ZCK_LOG_WARNING,
+        "Chunk %llu not found",
+        (long long unsigned) number
+    );
     return NULL;
 }
 
-zckChunk PUBLIC *zck_get_first_chunk(zckCtx *zck) {
+zckChunk ZCK_PUBLIC_API *zck_get_first_chunk(zckCtx *zck) {
     VALIDATE_PTR(zck);
 
     return zck->index.first;
 }
 
-zckChunk PUBLIC *zck_get_next_chunk(zckChunk *idx) {
+zckChunk ZCK_PUBLIC_API *zck_get_next_chunk(zckChunk *idx) {
     if(idx && idx->zck) {
         VALIDATE_PTR(idx->zck);
         ALLOCD_PTR(idx->zck, idx);
@@ -157,7 +189,18 @@ zckChunk PUBLIC *zck_get_next_chunk(zckChunk *idx) {
     return idx->next;
 }
 
-ssize_t PUBLIC zck_get_chunk_start(zckChunk *idx) {
+zckChunk ZCK_PUBLIC_API *zck_get_src_chunk(zckChunk *idx) {
+    if(idx && idx->zck) {
+        VALIDATE_PTR(idx->zck);
+        ALLOCD_PTR(idx->zck, idx);
+    } else {
+        ALLOCD_PTR(NULL, idx);
+    }
+
+    return idx->src;
+}
+
+ssize_t ZCK_PUBLIC_API zck_get_chunk_start(zckChunk *idx) {
     if(idx && idx->zck) {
         VALIDATE_INT(idx->zck);
         ALLOCD_INT(idx->zck, idx);
@@ -171,7 +214,7 @@ ssize_t PUBLIC zck_get_chunk_start(zckChunk *idx) {
         return idx->start;
 }
 
-ssize_t PUBLIC zck_get_chunk_size(zckChunk *idx) {
+ssize_t ZCK_PUBLIC_API zck_get_chunk_size(zckChunk *idx) {
     if(idx && idx->zck) {
         VALIDATE_INT(idx->zck);
         ALLOCD_INT(idx->zck, idx);
@@ -182,7 +225,7 @@ ssize_t PUBLIC zck_get_chunk_size(zckChunk *idx) {
     return idx->length;
 }
 
-ssize_t PUBLIC zck_get_chunk_comp_size(zckChunk *idx) {
+ssize_t ZCK_PUBLIC_API zck_get_chunk_comp_size(zckChunk *idx) {
     if(idx && idx->zck) {
         VALIDATE_INT(idx->zck);
         ALLOCD_INT(idx->zck, idx);
@@ -193,7 +236,7 @@ ssize_t PUBLIC zck_get_chunk_comp_size(zckChunk *idx) {
     return idx->comp_length;
 }
 
-ssize_t PUBLIC zck_get_chunk_number(zckChunk *idx) {
+ssize_t ZCK_PUBLIC_API zck_get_chunk_number(zckChunk *idx) {
     if(idx && idx->zck) {
         VALIDATE_INT(idx->zck);
         ALLOCD_INT(idx->zck, idx);
@@ -204,7 +247,7 @@ ssize_t PUBLIC zck_get_chunk_number(zckChunk *idx) {
     return idx->number;
 }
 
-int PUBLIC zck_get_chunk_valid(zckChunk *idx) {
+int ZCK_PUBLIC_API zck_get_chunk_valid(zckChunk *idx) {
     if(idx && idx->zck) {
         VALIDATE_INT(idx->zck);
         ALLOCD_INT(idx->zck, idx);
@@ -215,7 +258,7 @@ int PUBLIC zck_get_chunk_valid(zckChunk *idx) {
     return idx->valid;
 }
 
-bool PUBLIC zck_compare_chunk_digest(zckChunk *a, zckChunk *b) {
+bool ZCK_PUBLIC_API zck_compare_chunk_digest(zckChunk *a, zckChunk *b) {
     if(a && a->zck) {
         VALIDATE_BOOL(a->zck);
         ALLOCD_BOOL(a->zck, a);
@@ -236,7 +279,7 @@ bool PUBLIC zck_compare_chunk_digest(zckChunk *a, zckChunk *b) {
     return true;
 }
 
-int PUBLIC zck_missing_chunks(zckCtx *zck) {
+int ZCK_PUBLIC_API zck_missing_chunks(zckCtx *zck) {
     VALIDATE_READ_INT(zck);
 
     int missing = 0;
@@ -246,7 +289,7 @@ int PUBLIC zck_missing_chunks(zckCtx *zck) {
     return missing;
 }
 
-int PUBLIC zck_failed_chunks(zckCtx *zck) {
+int ZCK_PUBLIC_API zck_failed_chunks(zckCtx *zck) {
     VALIDATE_READ_INT(zck);
 
     int failed = 0;
@@ -256,7 +299,7 @@ int PUBLIC zck_failed_chunks(zckCtx *zck) {
     return failed;
 }
 
-void PUBLIC zck_reset_failed_chunks(zckCtx *zck) {
+void ZCK_PUBLIC_API zck_reset_failed_chunks(zckCtx *zck) {
     if(!zck)
         return;
 
@@ -265,3 +308,29 @@ void PUBLIC zck_reset_failed_chunks(zckCtx *zck) {
             idx->valid = 0;
     return;
 }
+
+bool ZCK_PUBLIC_API zck_generate_hashdb(zckCtx *zck) {
+    if (zck->index.ht || zck->index.htuncomp) {
+        zck_log(ZCK_LOG_ERROR, "Hash DB already present, it could not be created");
+        return false;
+    }
+
+    for(zckChunk *idx = zck->index.first; idx; idx=idx->next) {
+        zckChunk *tmp = NULL;
+        HASH_FIND(hh, zck->index.ht, idx->digest, idx->digest_size, tmp);
+        if(!tmp)
+            HASH_ADD_KEYPTR(hh, zck->index.ht, idx->digest, idx->digest_size,
+                            idx);
+        /*
+         * Do the same if there is uncompressed digest
+         */
+        if (zck->has_uncompressed_source && idx->digest_uncompressed) {
+            HASH_FIND(hhuncomp, zck->index.htuncomp, idx->digest_uncompressed, idx->digest_size, tmp);
+            if(!tmp)
+               HASH_ADD_KEYPTR(hhuncomp, zck->index.htuncomp, idx->digest_uncompressed, idx->digest_size,
+                               idx);
+        }
+    }
+
+    return true;
+}
